/* ******************************************************************************

	FILE:			SSTexture.cpp
	AUTHOR:			Jean-Sebastien Perrier

	DESCRIPTION:	OpenGL wrapper for texture management.

****************************************************************************** */

// Includes ****************************************************************** //
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glaux.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>


#include "SSTexture.h"

// Defines ******************************************************************* //

// Types ********************************************************************* //

// Consts ******************************************************************** //

// Globals ******************************************************************* //

// Static Members Storage Point ********************************************** //

SSTexture *SSTexture::s_pLastApplied = NULL;

// Function Prototypes ******************************************************* //

static AUX_RGBImageRec *LoadBMP(const char *Filename);

// Module External Functions ************************************************* //

SSTexture::SSTexture(const char *a_szFilename, bool a_bMipmap) : 
m_bLoaded(false),
m_bMipmap(a_bMipmap),
m_unTextureID(0)
{
	strcpy(m_szFilename, a_szFilename);
	m_bLoaded = LoadImage();
}


SSTexture::~SSTexture()
{
	if(m_bLoaded)
	{	// delete the texture from the driver
		glDeleteTextures(1, &m_unTextureID);
	}
}


/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	Apply
// DESCRIPTION:	Applies (bind) the texture to the device driver.
//				State caching is performed in order to prevent
//				binding the same texture over and over again. This
//				may cause a slow down on some graphic cards.
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSTexture::Apply()
{
	if(s_pLastApplied != this)
	{
		glBindTexture(GL_TEXTURE_2D, m_unTextureID);
		s_pLastApplied = this;
	}
}

// Module Internal Functions ************************************************* //

bool SSTexture::LoadImage()
{
	AUX_RGBImageRec *pImage = LoadBMP(m_szFilename);
	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if(pImage)
	{
		glGenTextures(1, &m_unTextureID);					// Create The Texture

		// Typical Texture Generation Using Data From The Bitmap
		glBindTexture(GL_TEXTURE_2D, m_unTextureID);
	
		if(m_bMipmap)
		{
			// Trilinear filtering (uses mipmaps).
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, pImage->sizeX, pImage->sizeY, GL_RGB, GL_UNSIGNED_BYTE, pImage->data);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		}
		else
		{
			// Bilinear filtering (no mipmap, less memory required, better for older cards).
			glTexImage2D(GL_TEXTURE_2D, 0, 3, pImage->sizeX, pImage->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, pImage->data);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		}

		// The texture is binded to the driver, so we can delete the image itself.
		free(pImage->data);
		free(pImage);
		return true;
	}
	return false;
}



AUX_RGBImageRec *LoadBMP(const char *a_szFilename)
{
	FILE *File=NULL;				// File Handle
	if(!a_szFilename)	return NULL;

	File = fopen(a_szFilename,"r");	// Check To See If The File Exists
	if(File)						// Does The File Exist?
	{
		fclose(File);				// Close The Handle
		return auxDIBImageLoad(a_szFilename);
	}
	return NULL;
}
